package com.inteligr8.github.http;

import java.io.IOException;

import org.apache.http.HttpException;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpRequestInterceptor;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.AuthState;
import org.apache.http.auth.Credentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpCoreContext;

/**
 * This class allows the developer to force the sending of credentials on the
 * first HTTP request.  Without this interceptor, all requests will first try
 * without credentials.
 * 
 * A subsequent retry will happen only under certain HTTP status codes.  Some
 * services will return a 404 instead of an expected 401 or 403.  This is for
 * security reasons.  As a side effect, it fools the Apache HTTP client into
 * thinking that the resource does not exist, even if it were authorized.  So
 * it will not retry it with credentials.
 *  
 * @author brian@inteligr8.com
 */
public class PreemptiveAuthInterceptor implements HttpRequestInterceptor {

	/**
	 * @see HttpRequestInterceptor#process(HttpRequest, HttpContext)
	 */
    public void process(final HttpRequest request, final HttpContext context) throws HttpException, IOException {
        AuthState authState = (AuthState) context.getAttribute(HttpClientContext.TARGET_AUTH_STATE);

        // If no auth scheme available yet, try to initialize it
        // preemptively
        if (authState.getAuthScheme() == null) {
            CredentialsProvider credsProvider = (CredentialsProvider) context.getAttribute(HttpClientContext.CREDS_PROVIDER);
            HttpHost targetHost = (HttpHost) context.getAttribute(HttpCoreContext.HTTP_TARGET_HOST);
            Credentials creds = credsProvider.getCredentials(new AuthScope(targetHost.getHostName(), targetHost.getPort()));
            if (creds == null) {
                throw new HttpException("No credentials for preemptive authentication");
            }
            authState.update(new BasicScheme(), creds);
        }

    }
	
}
